import { Contract, ethers } from 'ethers';
import { networkConfigs } from './utils/wallet-helper';
import { _abi } from './contracts/ERC721Offsetable';
export var MintStatus;
(function (MintStatus) {
  MintStatus[MintStatus["CLOSED"] = 0] = "CLOSED";
  MintStatus[MintStatus["PUBLIC"] = 1] = "PUBLIC";
  MintStatus[MintStatus["PRIVATE"] = 2] = "PRIVATE";
  MintStatus[MintStatus["BOTH"] = 3] = "BOTH";
})(MintStatus || (MintStatus = {}));
export class Block {
  constructor(contractAddress, testnet, type, walletInfo) {
    this.contractAddress = contractAddress;
    this.testnet = testnet;
    this.size = 0;
    this.contract = {};
    this.walletInfo = walletInfo;
    this.privatePrice = '0';
    this.publicPrice = '0';
    this.type = type;
    this.status = MintStatus.CLOSED;
    this.personalAllocationLeft = 0;
    this.tokensOwned = 0;
    this.isInitialised = false;
  }
  async init() {
    try {
      if (this.type === 'ERC721') {
        if (!this.contractAddress.match(/^0x[a-fA-F0-9]{40}$/)) {
          throw new Error('Missing or invalid contract address');
        }
        await this._checkChain();
        await this._getContractDetails();
        this.isInitialised = true;
        window.ethereum.on('disconnect', () => {
          console.log('Disconnected');
          window.location.reload();
        });
        window.ethereum.on('accountsChanged', wallets => {
          console.log(wallets);
          window.location.reload();
        });
      }
    }
    catch (error) {
      console.log(error);
    }
  }
  static async create(contractAddress, testnet, type, walletInfo) {
    try {
      const block = new Block(contractAddress, testnet, type, walletInfo);
      await block.init();
      return block;
    }
    catch (error) {
      return null;
    }
  }
  async mint(quantity) {
    if (this.isPublicMint()) {
      const priceInEth = ethers.utils.formatEther(this.publicPrice);
      const totalEth = ethers.utils.parseEther(priceInEth).mul(quantity);
      return this._publicMint(quantity, totalEth);
    }
    else if (this.isPrivateMint()) {
      const amountInEth = ethers.utils.formatEther(this.privatePrice);
      const totalEth = ethers.utils.parseEther(amountInEth).mul(quantity);
      return this._privateMint(quantity, totalEth);
    }
  }
  async _publicMint(quantity, amount) {
    const estimate = await this.contract.estimateGas.publicMint(quantity, { value: amount });
    const safeGasLimit = Math.floor(parseInt(estimate.toString()) * 1.2);
    return this.contract.publicMint(quantity, { gasLimit: safeGasLimit, value: amount });
  }
  async _privateMint(quantity, amount) {
    const estimate = await this.contract.estimateGas.privateMint(quantity, this.walletSignature, { value: amount });
    const safeGasLimit = Math.floor(parseInt(estimate.toString()) * 1.2);
    return this.contract.privateMint(quantity, this.walletSignature, { gasLimit: safeGasLimit, value: amount });
  }
  async _getContractDetails() {
    try {
      this.contract = new Contract(this.contractAddress, _abi, this.walletInfo.signer);
      if (!this.contract) {
        throw new Error('Contract initialization failed.');
      }
      const config = await this.contract.config();
      const { maxSupply, maxPerAddress, status, privatePrice, publicPrice } = config;
      // set the current mint progress
      this.totalSupply = (await this.contract.totalSupply()).toNumber();
      this.maxSupply = maxSupply;
      // check how many were minted from the user address
      const filter = this.contract.filters.Transfer('0x0000000000000000000000000000000000000000', this.walletInfo.signerAddress);
      let events = await this.contract.queryFilter(filter);
      this.personalAllocationLeft = Math.max(0, maxPerAddress - events.length);
      this.publicPrice = publicPrice.toString();
      this.privatePrice = privatePrice.toString();
      this.maxPerAddress = maxPerAddress;
      this.status = status;
    }
    catch (error) {
      console.error(error);
    }
  }
  isPublicMint() {
    return this.status === MintStatus.PUBLIC || (this.status === MintStatus.BOTH && !this.walletSignature);
  }
  isPrivateMint() {
    return this.status === MintStatus.PRIVATE || (this.status === MintStatus.BOTH && this.walletSignature);
  }
  async _checkChain() {
    const requiredNetwork = this.testnet ? networkConfigs.goerli : networkConfigs.homestead;
    if (!((await this.walletInfo.provider.getNetwork()).name === requiredNetwork.name)) {
      try {
        await window.ethereum.request({
          method: 'wallet_switchEthereumChain',
          params: [{ chainId: requiredNetwork.chainId }], // chainId must be in hexadecimal numbers
        });
        window.location.reload();
      }
      catch (error) {
        console.error(error);
      }
    }
  }
}
